/*
	Xiliinx ISP parallel port I/F Version 0.5

	Copyright(c) 2002 Tatsuyuki Satoh

	2001. 3.18 1st release

	XilinxpP[uɓKp
	^[Qbgubb̃`FbN@\

	I/O𒼐ڂ̂ŁAWindowsNT/200/XPł͓삵܂B
	Windows2000pphCoW

	PDoE_XL(JTAG)[h}

	D0    -> TDI
	D1    -> TCK
	D2    -> TMS
	D3    -> -ENABLE TDI,TCK,TMS
	D4    -> -ENABLE TDO=L (for VCC sense)
	D6    -> PROBLE SENSE(to PE & BUSY)
	SLCT  <- TDI
	ERROR <- TargetVCC
	BUSY  <- PROBLE SENSE(from D6)
	PE    <- PROBLE SENSE(from D6)

	QDVAX[u[h}

	D0    -> DIN
	D1    -> CCLK
	D2    -> PROGRAM
	D3    -> -ENABLE DIN,CCLK,PROGARM
	D4    -> -ENABLE DONE=L (for VCC sense)
	D6    -> PROBLE SENSE(to PE & BUSY)
	SLCT  <- DONE
	ERROR <- TargetVCC
	BUSY  <- PROBLE SENSE(from D6)
	PE    <- PROBLE SENSE(from D6)

	note:

	@RpC̓t[̃RpCAMINGWgpĂ܂B

*/
#include <windows.h>
#include <stdio.h>

/* hCo[\ */
#include "isp_if.h"
#include "isp_par.h"

int par_jtag_init(DWORD port_address);
int par_jtag_power(void);
int par_jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e);
void par_jtag_end(void);

int par_ss_init(DWORD port_address);
int par_ss_power(void);
int par_ss_tx(const BYTE *data,DWORD size);
int par_ss_program(BYTE level);
int par_ss_done(void);
void par_ss_end(void);

/* -------------------- Driver structure -------------------- */
ISP_PROBE par_probe =
{
//JTAG
	par_jtag_init,
	par_jtag_power,
	par_jtag_txrx,
	par_jtag_end,
//SS
	par_ss_init,
	par_ss_power,
	par_ss_tx,
	par_ss_program,
	par_ss_done,
	par_ss_end
};
/* ---------------------------------------------------------- */

//ph^egptO@^@h^nAhX
static DWORD par_port;

/* I/OANZX̃}N */

/* macro : output I/O port */
__inline__ void outportb (unsigned short _port, unsigned char _data)
{
  __asm__ __volatile__ ("outb %1, %0" : : "dN" (_port),"a" (_data));
}

/* macro : input I/O port */
__inline__ unsigned char inportb (unsigned short _port)
{
  unsigned char rv;
  __asm__ __volatile__ ("inb %1, %0" : "=a" (rv) : "dN" (_port));
  return rv;
}

// JTAG out 1clock
#define PAR_JTAG(TMS,TDI) {\
	outportb(par_port,(TMS?4:0) | (TDI?1:0) | 0x10); \
	outportb(par_port,(TMS?4:0) | (TDI?1:0) | 0x12); \
}

// JTAG TDO sense
#define PAR_TDO() ((inportb(par_port+1)>>4)&1)

// VCC sense
#define PAR_VCC() ((inportb(par_port+1)>>3)&1)
// probe sense
#define PAR_PR0() ((inportb(par_port+1)>>5)&1)
#define PAR_PR1() ((inportb(par_port+1)>>7)&1)

//p|[g̏
int par_jtag_init(DWORD port_address)
{
	par_port = port_address;

	printf("Parallel I/F port=%04X Probe Initialize\n",port_address);

	// probe sense
	outportb(par_port,0x50);
	if( !PAR_PR0() || PAR_PR1() )
	{
		printf("No Probe\n");
		return 1;
	}
	outportb(par_port,0x10);
	if( PAR_PR0() || !PAR_PR1() )
	{
		printf("No Probe\n");
		return 1;
	}

	return 0;
}

//d`FbN
int par_jtag_power(void)
{
	return  PAR_VCC() ? 1: 0;
}

//is`fɂ鑗M
int par_jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
{
	int c;

	// clear outbuf
	if(obuf)
		for(c=0;c<(drsize+7)/8;c++)
			obuf[c] = 0;

	for(c=0;c<drsize;c++)
	{
		int tdi = ibuf && (ibuf[c/8]&(0x80>>(c&7))) ? 1 : 0;
		int tms = (c==drsize-1) ? tms_e : tms_s;
		// out 1 TCK
		PAR_JTAG(tms,tdi);
		// TDO get
		if( obuf && PAR_TDO() )
			obuf[c/8] |= 0x01<<(c&7);
	}
	return 0;
}

void par_jtag_end(void)
{
}

/*-------------------------------------------------------------------------------*/
#define CF_DIN  0x01
#define CF_CCLK 0x02
#define CF_PROGRAM 0x04
#define CF_BASE (0x10|CF_PROGRAM)

#define PAR_DONE() ((inportb(par_port+1)>>4)&1)

int par_ss_init(DWORD port_address)
{
	return par_jtag_init(port_address);
}

int par_ss_power(void)
{
	return par_jtag_power();
}
int par_ss_tx(const BYTE *data,DWORD size)
{
	int pos;

	// PubN̑Mf[^쐬
	for(pos=0 ; pos<size ; pos++)
	{
		BYTE value  = data[pos];

		BYTE outval;
		outval = (value&0x80) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x40) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x20) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x10) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x08) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x04) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x02) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
		outval = (value&0x01) ? CF_BASE | CF_DIN : CF_BASE;
		outportb(par_port,outval);
		outportb(par_port,outval|CF_CCLK);
	}
	return 0;


	return 1;
}
int par_ss_program(BYTE level)
{
	outportb(par_port,0x10);
	outportb(par_port,0x10);
	outportb(par_port,CF_BASE);
	return 0;
}
int par_ss_done(void)
{
	return PAR_DONE();
}
void par_ss_end(void)
{
}
