/*
	Xiliinx BitStream File Handler Ver.0.9

	Copyright(c) 2002 Tatsuyuki Satoh

*/
#include <windows.h>
#include <stdio.h>

#include "xilinxbit.h"

// Header ID
#define ID_NETNAME 0x61
#define ID_TAGET   0x62
#define ID_DATE    0x63
#define ID_TIME    0x64
#define ID_BITDATA 0x65

//Unknown File Header Data
const static BYTE unknown_header[13] = 
{ 0x00,0x09,0x0f,0xf0,0x0f,0xf0,0x0f,0xf0,0x0f,0xf0,0x00,0x00,0x01 };

//t@C̃I[vƁAwb_[̉
BIT_FILE *bitfile_open(char *filepath)
{
	BIT_FILE *p;
	HANDLE hndl;
	void *ptr;
	BYTE buf[4];
	DWORD read_size;
	// file open
	hndl = CreateFile(filepath,GENERIC_READ,
		0,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
	if( hndl==INVALID_HANDLE_VALUE)
		return NULL;
	// memory allocate & struct init
	p = (BIT_FILE *)malloc(sizeof(BIT_FILE));
	if( p==NULL )
	{
		CloseHandle(hndl);
		return NULL;
	}
	p->hndl     = hndl;
	p->bit_size = 0;
	p->netname[0] = p->targetname[0] = p->date[0] = p->time[0] = 0;
	//wb_̃[hƉ
	if( !ReadFile(hndl,p->unknown,13,&read_size,NULL) )
	{
		free(p);
		CloseHandle(hndl);
		return NULL;
	}
	while(1)
	{
		BYTE id;
		if( !ReadFile(hndl,&id,1,&read_size,NULL) )
			break;
		switch( id )
		{
		case ID_NETNAME: ptr = &p->netname; break;
		case ID_TAGET:   ptr = &p->targetname; break;
		case ID_DATE:    ptr = &p->date;  break;
		case ID_TIME:    ptr = &p->time;  break;
		case ID_BITDATA:
			if(!ReadFile(hndl,buf,4,&read_size,NULL))
				break;
			p->bit_size = (DWORD)buf[0]*0x1000000 +
						  (DWORD)buf[1]*0x10000 +
						  (DWORD)buf[2]*0x100 +
						  (DWORD)buf[3];
			return p;
		default:
			printf("Unknown header ID %02x found\n",id);
			break;
		}
		/* word size string load */
		if(!ReadFile(hndl,buf,2,&read_size,NULL))
			break;
		if(!ReadFile(hndl,ptr,(DWORD)buf[0]*0x100 + buf[1],&read_size,NULL))
			break;
	}
	// error
	free(p);
	CloseHandle(hndl);
	return NULL;
}

//t@C̃[h
DWORD bitfile_read(BIT_FILE *hndl,void *buf,DWORD size)
{
	DWORD read_size;
	if(!ReadFile(hndl->hndl,buf,size,&read_size,NULL))
		return 0;
	return read_size;
}

//t@C̃N[Y
void bitfile_close(BIT_FILE *hndl)
{
	CloseHandle(hndl->hndl);
	free(hndl);
}

